using System;

namespace LCUIssue
{
    public class DataRecord : IEquatable<DataRecord>, IComparable<DataRecord>, IComparable
    {
        public int CompareTo(DataRecord other)
        {
            if (ReferenceEquals(this, other)) return 0;
            if (ReferenceEquals(null, other)) return 1;
            return Time.CompareTo(other.Time);
        }

        public int CompareTo(object obj)
        {
            if (ReferenceEquals(null, obj)) return 1;
            if (ReferenceEquals(this, obj)) return 0;
            if (!(obj is DataRecord)) throw new ArgumentException($"Object must be of type {nameof(DataRecord)}");
            return CompareTo((DataRecord) obj);
        }

        public bool Equals(DataRecord other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return Time.Equals(other.Time) && Interval == other.Interval && Value.Equals(other.Value) && AuditStampId == other.AuditStampId;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != GetType()) return false;
            return Equals((DataRecord) obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = Time.GetHashCode();
                hashCode = (hashCode * 397) ^ Interval;
                hashCode = (hashCode * 397) ^ Value.GetHashCode();
                hashCode = (hashCode * 397) ^ AuditStampId;
                return hashCode;
            }
        }

        public static bool operator ==(DataRecord left, DataRecord right)
        {
            return Equals(left, right);
        }

        public static bool operator !=(DataRecord left, DataRecord right)
        {
            return !Equals(left, right);
        }

        public readonly DateTime Time;
        public readonly int Interval;
        public readonly double Value;
        public readonly int AuditStampId;

        public DataRecord(DateTime time, int interval, double value, int auditStampId)
        {
            Time = time;
            Interval = interval;
            Value = value;
            AuditStampId = auditStampId;
        }
    }
}