﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Arction.WinForms.Charting;
using Arction.WinForms.Charting.Axes;
using Arction.WinForms.Charting.SeriesXY;
using Arction.WinForms.Charting.Titles;

namespace LCUIssue
{
    public partial class Form1 : Form
    {
        private LightningChartUltimate _chart;

        public Form1()
        {
            InitializeComponent();
        }

        private SortedSet<DataRecord> GetArtificialSeries(int datapointCount = 250000)
        {
            var cache = new List<DataRecord>();
            var validDatapointCount = 5;
            var seed = datapointCount / validDatapointCount;
            var start = DateTime.Now;
            for (var i = 0; i < datapointCount; i++)
            {
                cache.Add(new DataRecord(start.AddMinutes(i * 5), 300, (i % seed == 0) ? new Random().NextDouble() : double.NaN, 0));
            }
            cache.Sort();
            return new SortedSet<DataRecord>(cache);
        }

        private void CreateChart(bool filterNaN, bool showLine, bool showMarkers)
        {

            // Dispose if the chart already existed - had performance issues also trying to update an existing chart...
            _chart?.Dispose();

            // Create the new chart component
            _chart = new LightningChartUltimate();

            // Start the update process - this disables any screen update while we set up everything
            _chart.BeginUpdate();

            _chart.Parent = panel2;
            _chart.Dock = DockStyle.Fill;
            _chart.BringToFront();
            _chart.Name = _chart.Title.Text = "Some Title";
            _chart.Title.Visible = false;
            _chart.ColorTheme = ColorTheme.LightGray;
            _chart.Background.Style = RectFillStyle.None;

            // Setup legend
            _chart.ViewXY.AutoSpaceLegendBoxes = true;
            var legendBox = _chart.ViewXY.LegendBoxes[0];
            legendBox.Position = LegendBoxPositionXY.TopCenter;
            legendBox.BorderWidth = 1;
            legendBox.BorderColor = Color.DimGray;
            legendBox.Shadow.Visible = false;
            legendBox.Fill.Style = RectFillStyle.None;
            legendBox.HighlightSeriesOnTitle = false;
            legendBox.AllowMouseResize = false;
            legendBox.MoveByMouse = false;
            legendBox.SeriesTitleFont = new Font(legendBox.SeriesTitleFont.FontFamily, 8.25f);
            legendBox.SeriesTitleColor = Color.Black;
            legendBox.UnitsColor = Color.DimGray;
            legendBox.UnitsFont = new Font(legendBox.UnitsFont.FontFamily, 8.25f);
            legendBox.CheckBoxColor = Color.Transparent;

            var itemData = GetArtificialSeries();

            // Get complete time range across all series to build the X-Axis
            var startTime = itemData.First().Time;
            var endTime = itemData.Last().Time;

            //Setup x-axis
            AxisX xAxis = _chart.ViewXY.XAxes[0];
            xAxis.DateOriginYear = startTime.Year;
            xAxis.DateOriginMonth = startTime.Month;
            xAxis.DateOriginDay = startTime.Day;
            xAxis.SetDateTimeRange(startTime, endTime);
            xAxis.ScrollMode = XAxisScrollMode.None;
            xAxis.ValueType = AxisValueType.DateTime;

            xAxis.Title.Visible = false;
            xAxis.LabelsFont = new Font(xAxis.LabelsFont.FontFamily, 8.25f);
            xAxis.AutoFormatLabels = true;
            xAxis.LabelsAngle = 0;
            xAxis.ScrollMode = XAxisScrollMode.None;
            xAxis.LabelTicksGap = 0;
            xAxis.MinorGrid.Visible = true;
            // Place at bottom
            xAxis.Position = 100;

            //Setup y-axes
            _chart.ViewXY.AxisLayout.YAxisAutoPlacement = YAxisAutoPlacement.LeftThenRight;

            AxisY yAxis = _chart.ViewXY.YAxes[0];
            yAxis.AxisColor = Color.Black;
            yAxis.Title.Text = "Something";
            yAxis.Units.Text = "Cnt";
            yAxis.Reversed = false;
            yAxis.Minimum = 0;
            yAxis.Maximum = 1;

            yAxis.LabelsFont = new Font(xAxis.LabelsFont.FontFamily, 8.25f);
            yAxis.LabelsColor = Color.Black;
            yAxis.Title.Shadow.Style = TextShadowStyle.Off;
            yAxis.Title.Font = new Font(xAxis.LabelsFont.FontFamily, 9f);
            yAxis.Title.Color = yAxis.AxisColor;
            yAxis.Units.Color = Color.Black;
            yAxis.Units.Angle = 90;
            yAxis.Units.VerticalAlign = YAxisTitleAlignmentVertical.Top;
            yAxis.Units.Visible = true;
            yAxis.Units.Font = new Font(yAxis.Units.Font.FontFamily, 9f);
            yAxis.Visible = true;

            // Apply major gridlines
            yAxis.MajorGrid.Color = Color.DarkGray;
            yAxis.MajorGrid.Pattern = LinePattern.Dot;
            yAxis.MajorGrid.PatternScale = 1;
            yAxis.MajorGrid.Visible = true;

            // Apply minor gridlines
            yAxis.MinorGrid.Color = Color.LightGray;
            yAxis.MinorGrid.Pattern = LinePattern.SmallDot;
            yAxis.MinorGrid.PatternScale = 1;
            yAxis.MinorGrid.Visible = false;

            // Set up major tick marks
            yAxis.AutoDivSpacing = true;
            yAxis.MajorDivCount = 5;
            yAxis.MajorDivTickStyle.Color = Color.Black;
            yAxis.MajorDivTickStyle.LineLength = 8;
            yAxis.MajorDivTickStyle.Visible = true;
            yAxis.MajorDivTickStyle.Alignment = Alignment.Near;

            // Set up minor tick marks
            yAxis.MinorDivCount = 4;
            yAxis.MinorDivTickStyle.Color = Color.Gray;
            yAxis.MinorDivTickStyle.LineLength = 3;
            yAxis.MinorDivTickStyle.Visible = false;
            yAxis.MinorDivTickStyle.Alignment = Alignment.Near;


            // Final options
            _chart.ResetPageSettings();
            _chart.ViewXY.ZoomToFit();
            _chart.ViewXY.ClearZoomStack();



            // Use right-to-left box selection as Zoom Undo instead of the default full-reset operation
            _chart.ViewXY.ZoomPanOptions.RightToLeftZoomAction = RightToLeftZoomActionXY.PopFromZoomStack;

            // Clear any pre-existing data series
            _chart.ViewXY.PointLineSeries.Clear();

            // Define new series
            var mos = new PointLineSeries(_chart.ViewXY, xAxis, yAxis)
            {
                PointsVisible = showMarkers,
                Points = filterNaN
                    ? itemData.Where(x => !double.IsNaN(x.Value)).Select(x => new SeriesPoint(xAxis.DateTimeToAxisValue(x.Time), x.Value, x.AuditStampId)).ToArray()
                    : itemData.Select(x => new SeriesPoint(xAxis.DateTimeToAxisValue(x.Time), x.Value, x.AuditStampId)).ToArray(),
                LineVisible = showLine,
                LineStyle = new LineStyle
                {
                  Color  = Color.Black,
                  Width = 1,
                  Pattern = LinePattern.Solid,
                },
                PointStyle = new PointShapeStyle
                {
                    BorderWidth = 0,
                    GradientFill = GradientFillPoint.Solid,
                    Color1 = Color.Black,
                    BorderColor = Color.Black,
                    Width = 5,
                    Height = 5,
                    Angle = 0,
                    Shape = Shape.Rectangle,
                },
                Title = new SeriesTitle
                {
                    Text = "Something"
                },
                MouseInteraction = false,
                DataBreaking =
                {
                    Enabled = true,
                    Value = double.NaN
                },
                CursorTrackEnabled = false,
                ShowInLegendBox = true,
            };

            // Add the series to the chart
            _chart.ViewXY.PointLineSeries.Add(mos);

            // We're done with all our updates - the chart may now be painted on the screen
            _chart.EndUpdate();

            // Apply auto-fit as needed
            bool changed;
            yAxis.Fit(5d, out changed, true, true);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            CreateChart(cbFilterNaN.Checked, cbShowLine.Checked, cbShowMarkers.Checked);
        }

    }
}
